==> hgwebplus-0.1.0.tar.gz: hgwebplus-0.1.0/src/hgwebplus.py <==
# vi:et:ts=4 sw=4 sts=4
#
# hgwebplus
# Copyright (c) 2020 Gary Kramlich <grim@reaperworld.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with hgwebplus.  If not, see <https://www.gnu.org/licenses/>.

import re

from hashlib import md5 as hashlib_md5

from mercurial import (
    extensions,
    match,
    mdiff,
    patch,
    registrar,
    templateutil
)
from mercurial.hgweb import webutil, hgweb_mod
from mercurial.hgweb.common import paritygen

import cmarkgfm

filters = {}
templatefilter = registrar.templatefilter(filters)


@templatefilter(b'md5', intype=bytes)
def md5(text):
    """ md5 is a template filter that will return an MD5 hash for whatever text
        was given to it.  This is typically used to generate gravatar URLs.
    """
    return hashlib_md5(text).hexdigest().encode('ascii')


def _diffstattmplgen(orig, context, ctx, statgen, parity):
    """ This is a reimplementation of hgweb.webutil._diffstattmplgen.  It adds
        the number lines add and removed as well as if the file is binary or
        not.

        This allows you to output something like the following instead of just
        having percentages.

            file1.txt +10 -2
            file2.txt +20 -0
    """
    stats, maxname, maxtotal, addtotal, removetotal, binary = next(statgen)
    files = ctx.files()

    def pct(i):
        if maxtotal == 0:
            return 0
        return (float(i) / maxtotal) * 100

    fileno = 0
    for filename, adds, removes, isbinary in stats:
        template = b'diffstatlink' if filename in files else b'diffstatnolink'
        total = adds + removes
        fileno += 1

        data = {
            b'node': ctx.hex(),
            b'file': filename,
            b'fileno': fileno,
            b'total': total,
            b'added': adds,
            b'addpct': pct(adds),
            b'removed': removes,
            b'removepct': pct(removes),
            b'parity': next(parity),
            b'binary': isbinary,
        }

        yield context.process(template, data)


# This regular expression captures the start line numbers from the hunk token
# for a diff.
RE_DIFF = re.compile(r'^@@ -(\d+),\d+ \+(\d+),\d+ @@')


def _prettyprintdifflines(context, lines, blockno, lineidprefix, filea, reva,
                          fileb, revb):
    """ This function is an updated version of
        hgweb.webutil._prettyprintdifflines.  It maintains backwards
        compatibility but adds a lot of new features.  This is not directly
        wrapped, but is instead called from _diffsgen below, which is wrapped.

        This version provides additional details about each file.  The filenames
        are in filea, and fileb; The line number that's being displayed for both
        filea and fileb are in linea and lineb.  And finally the revision of
        each file is available in reva and revb.
    """
    linea = 0
    lineb = 0

    seena = False
    seenb = False

    for lineno, l in enumerate(lines, 1):
        showa = True
        showb = True

        match = RE_DIFF.match(l)
        if match is not None:
            ltype = b'difflineat'
            groups = match.groups()
            linea = int(groups[0])
            lineb = int(groups[1])
            showa = False
            showb = False
            seena = False
            seenb = False
        elif l.startswith(b'---'):
            ltype = b'difflineminus'
            showa = False
            showb = False
        elif l.startswith(b'+++'):
            ltype = b'difflineplus'
            showa = False
            showb = False
        elif l.startswith(b'-'):
            # only increment if we've already seen a minus line
            if seena:
                linea += 1
            ltype = b'difflineminus'
            showb = False
            seena = True
        elif l.startswith(b'+'):
            # only increment if we've already seen a plus line
            if seenb:
                lineb += 1
            ltype = b'difflineplus'
            showa = False
            seenb = True
        else:
            # only increment if we've already seen a plus/minus line
            if seena:
                linea += 1
            if seenb:
                lineb += 1
            seena = True
            seenb = True

            ltype = b'diffline'

        difflineno = b"%d.%d" % (blockno, lineno)

        data = {
            b'filea': filea,
            b'fileb': fileb,
            b'line': l,
            b'linea': showa and linea or b'',
            b'lineb': showb and lineb or b'',
            b'lineid': lineidprefix + b"l%s" % difflineno,
            b'lineno': lineno,
            b'linenumber': b"% 8s" % difflineno,
            b'reva': reva,
            b'revb': revb,
        }

        yield context.process(ltype, data)


def _diffsgen(
    orig,
    context,
    repo,
    ctx,
    basectx,
    files,
    style,
    stripecount,
    linerange,
    lineidprefix,
):
    """ this is a updated version of hgweb.webutil._diffsgen.  It maintains
        backwards compatibility but adds some additional values to the template
        context.  They are change type, filea, fileb, reva, revb.

        change type is one of modified, added, removed, or renamed.  filea and
        fileb are the respective file names.  Likewise, reva and revb are the
        respective revisions.
    """
    if files:
        m = match.exact(files)
    else:
        m = match.always()

    diffopts = patch.diffopts(repo.ui, untrusted=True)
    parity = paritygen(stripecount)

    diffhunks = patch.diffhunks(repo, basectx, ctx, m, opts=diffopts)
    for blockno, (fctx1, fctx2, header, hunks) in enumerate(diffhunks, 1):
        filea, fileb = b'/dev/null', b'/dev/null'
        reva, revb = b'', b''

        if fctx1 is not None:
            filea = fctx1.path()
            reva = fctx1.hex()

        if fctx2 is not None:
            fileb = fctx2.path()
            revb = fctx2.hex()

        if style != b'raw':
            header = header[1:]
        lines = [h + b'\n' for h in header]
        for hunkrange, hunklines in hunks:
            if linerange is not None and hunkrange is not None:
                s1, l1, s2, l2 = hunkrange
                if not mdiff.hunkinrange((s2, l2), linerange):
                    continue
            lines.extend(hunklines)

        args = (lines, blockno, lineidprefix, filea, reva, fileb, revb)
        l = templateutil.mappedgenerator(_prettyprintdifflines, args)

        changetype = b'modified'
        if filea == b'/dev/null':
            changetype = b'added'
        elif fileb == b'/dev/null':
            changetype = b'removed'
        elif filea != fileb:
            changetype = b'renamed'

        data = {
            b'blockno': blockno,
            b'changetype': changetype,
            b'filea': filea,
            b'fileb': fileb,
            b'lines': l,
            b'parity': next(parity),
            b'reva': reva,
            b'revb': revb,
        }

        yield data


def readme(context, mapping):
    """ readme is a template keyword that will attempt to render a readme file
        that is available in the repository.  It currently supports github
        flavored markdown and plain text.
    """

    ctx = context.resource(mapping, b'ctx')

    # we iterate the files instead of a fileset, because we want to
    # deterministically render readmes in the same order if there are more
    # than one in a repository.  With the fileset, we'd have to run through
    # the generator and call .lower on each item again, and then we'd lose
    # the ability to fallback if the prioritized one failed to render.
    for filename in ctx:
        lower = filename.lower()
        if lower == b'readme.md':
            raw_utf8 = ctx[filename].data().decode('utf-8')
            return cmarkgfm.markdown_to_html(raw_utf8).encode('utf-8')

        if lower in [b'readme.txt', b'readme', b'readme.md']:
            return ctx[filename].data().replace(b'\n', b'<br/>\n')


def repo_custom_templater(orig, self, req):
    """ this function is a wrapper of hgweb_mod.requestcontext.templater.  This
        is done to be able to add custom keywords into hgweb.  This is not an
        ideal solution, and in fact feels very hacky.  However, at the time of
        this writing, it appears to be the only way to do this.
    """
    templater = orig(self, req)

    templatekeyword = registrar.templatekeyword(templater._proc._defaults)

    templatekeyword(b'readme', requires={b'ctx', b'repo'})(readme)

    return templater


def extsetup(ui):
    extensions.wrapfunction(webutil, '_diffstattmplgen', _diffstattmplgen)
    extensions.wrapfunction(webutil, '_diffsgen', _diffsgen)

    # this nasty but amazingly working hack is from av6
    extensions.wrapfunction(
        hgweb_mod.requestcontext,
        'templater',
        repo_custom_templater,
    )

==> hgwebplus-0.2.0.tar.gz: hgwebplus-0.2.0/src/hgwebplus.py <==
# vi:et:ts=4 sw=4 sts=4
#
# hgwebplus
# Copyright (c) 2020 Gary Kramlich <grim@reaperworld.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with hgwebplus.  If not, see <https://www.gnu.org/licenses/>.

import re

from hashlib import md5 as hashlib_md5

from mercurial import (
    extensions,
    match,
    mdiff,
    patch,
    registrar,
    templateutil
)
from mercurial.hgweb import webutil, hgweb_mod
from mercurial.hgweb.common import paritygen

import cmarkgfm

filters = {}
templatefilter = registrar.templatefilter(filters)


@templatefilter(b'md5', intype=bytes)
def md5(text):
    """ md5 is a template filter that will return an MD5 hash for whatever text
        was given to it.  This is typically used to generate gravatar URLs.
    """
    return hashlib_md5(text).hexdigest().encode('ascii')


def _diffstattmplgen(orig, context, ctx, statgen, parity):
    """ This is a reimplementation of hgweb.webutil._diffstattmplgen.  It adds
        the number lines add and removed as well as if the file is binary or
        not.

        This allows you to output something like the following instead of just
        having percentages.

            file1.txt +10 -2
            file2.txt +20 -0
    """
    stats, maxname, maxtotal, addtotal, removetotal, binary = next(statgen)
    files = ctx.files()

    def pct(i):
        if maxtotal == 0:
            return 0
        return (float(i) / maxtotal) * 100

    fileno = 0
    for filename, adds, removes, isbinary in stats:
        template = b'diffstatlink' if filename in files else b'diffstatnolink'
        total = adds + removes
        fileno += 1

        data = {
            b'node': ctx.hex(),
            b'file': filename,
            b'fileno': fileno,
            b'total': total,
            b'added': adds,
            b'addpct': pct(adds),
            b'removed': removes,
            b'removepct': pct(removes),
            b'parity': next(parity),
            b'binary': isbinary,
        }

        yield context.process(template, data)


# This regular expression captures the start line numbers from the hunk token
# for a diff.
RE_DIFF = re.compile(r'^@@ -(\d+),\d+ \+(\d+),\d+ @@')


def _prettyprintdifflines(context, lines, blockno, lineidprefix, filea, reva,
                          fileb, revb):
    """ This function is an updated version of
        hgweb.webutil._prettyprintdifflines.  It maintains backwards
        compatibility but adds a lot of new features.  This is not directly
        wrapped, but is instead called from _diffsgen below, which is wrapped.

        This version provides additional details about each file.  The
        filenames are in filea, and fileb; The line number that's being
        displayed for both filea and fileb are in linea and lineb.  And finally
        the revision of each file is available in reva and revb.
    """
    linea = 0
    lineb = 0

    seena = False
    seenb = False

    for lineno, l in enumerate(lines, 1):
        showa = True
        showb = True

        match = RE_DIFF.match(l.decode('ascii'))
        if match is not None:
            ltype = b'difflineat'
            groups = match.groups()
            linea = int(groups[0])
            lineb = int(groups[1])
            showa = False
            showb = False
            seena = False
            seenb = False
        elif l.startswith(b'---'):
            ltype = b'difflineminus'
            showa = False
            showb = False
        elif l.startswith(b'+++'):
            ltype = b'difflineplus'
            showa = False
            showb = False
        elif l.startswith(b'-'):
            # only increment if we've already seen a minus line
            if seena:
                linea += 1
            ltype = b'difflineminus'
            showb = False
            seena = True
        elif l.startswith(b'+'):
            # only increment if we've already seen a plus line
            if seenb:
                lineb += 1
            ltype = b'difflineplus'
            showa = False
            seenb = True
        else:
            # only increment if we've already seen a plus/minus line
            if seena:
                linea += 1
            if seenb:
                lineb += 1
            seena = True
            seenb = True

            ltype = b'diffline'

        difflineno = b"%d.%d" % (blockno, lineno)

        data = {
            b'filea': filea,
            b'fileb': fileb,
            b'line': l,
            b'linea': showa and linea or b'',
            b'lineb': showb and lineb or b'',
            b'lineid': lineidprefix + b"l%s" % difflineno,
            b'lineno': lineno,
            b'linenumber': b"% 8s" % difflineno,
            b'reva': reva,
            b'revb': revb,
        }

        yield context.process(ltype, data)


def _diffsgen(
    orig,
    context,
    repo,
    ctx,
    basectx,
    files,
    style,
    stripecount,
    linerange,
    lineidprefix,
):
    """ this is a updated version of hgweb.webutil._diffsgen.  It maintains
        backwards compatibility but adds some additional values to the template
        context.  They are change type, filea, fileb, reva, revb.

        change type is one of modified, added, removed, or renamed.  filea and
        fileb are the respective file names.  Likewise, reva and revb are the
        respective revisions.
    """
    if files:
        m = match.exact(files)
    else:
        m = match.always()

    diffopts = patch.diffopts(repo.ui, untrusted=True)
    parity = paritygen(stripecount)

    diffhunks = patch.diffhunks(repo, basectx, ctx, m, opts=diffopts)
    for blockno, (fctx1, fctx2, header, hunks) in enumerate(diffhunks, 1):
        filea, fileb = b'/dev/null', b'/dev/null'
        reva, revb = b'', b''

        if fctx1 is not None:
            filea = fctx1.path()
            reva = fctx1.hex()

        if fctx2 is not None:
            fileb = fctx2.path()
            revb = fctx2.hex()

        if style != b'raw':
            header = header[1:]
        lines = [h + b'\n' for h in header]
        for hunkrange, hunklines in hunks:
            if linerange is not None and hunkrange is not None:
                s1, l1, s2, l2 = hunkrange
                if not mdiff.hunkinrange((s2, l2), linerange):
                    continue
            lines.extend(hunklines)

        args = (lines, blockno, lineidprefix, filea, reva, fileb, revb)
        prettylines = templateutil.mappedgenerator(_prettyprintdifflines, args)

        changetype = b'modified'
        if filea == b'/dev/null':
            changetype = b'added'
        elif fileb == b'/dev/null':
            changetype = b'removed'
        elif filea != fileb:
            changetype = b'renamed'

        data = {
            b'blockno': blockno,
            b'changetype': changetype,
            b'filea': filea,
            b'fileb': fileb,
            b'lines': prettylines,
            b'parity': next(parity),
            b'reva': reva,
            b'revb': revb,
        }

        yield data


def readme(context, mapping):
    """ readme is a template keyword that will attempt to render a readme file
        that is available in the repository.  It currently supports github
        flavored markdown and plain text.
    """

    ctx = context.resource(mapping, b'ctx')

    # we iterate the files instead of a fileset, because we want to
    # deterministically render readmes in the same order if there are more
    # than one in a repository.  With the fileset, we'd have to run through
    # the generator and call .lower on each item again, and then we'd lose
    # the ability to fallback if the prioritized one failed to render.
    for filename in ctx:
        lower = filename.lower()
        if lower == b'readme.md':
            raw_utf8 = ctx[filename].data().decode('utf-8')
            return cmarkgfm.markdown_to_html(raw_utf8).encode('utf-8')

        if lower in [b'readme.txt', b'readme', b'readme.md']:
            return ctx[filename].data().replace(b'\n', b'<br/>\n')


def repo_custom_templater(orig, self, req):
    """ this function is a wrapper of hgweb_mod.requestcontext.templater.  This
        is done to be able to add custom keywords into hgweb.  This is not an
        ideal solution, and in fact feels very hacky.  However, at the time of
        this writing, it appears to be the only way to do this.
    """
    templater = orig(self, req)

    templatekeyword = registrar.templatekeyword(templater._proc._defaults)

    templatekeyword(b'readme', requires={b'ctx', b'repo'})(readme)

    return templater


def extsetup(ui):
    extensions.wrapfunction(webutil, '_diffstattmplgen', _diffstattmplgen)
    extensions.wrapfunction(webutil, '_diffsgen', _diffsgen)

    # this nasty but amazingly working hack is from av6
    extensions.wrapfunction(
        hgweb_mod.requestcontext,
        'templater',
        repo_custom_templater,
    )
